# Copyright 2023 Qilimanjaro Quantum Tech
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""CONSTANTS"""


class REST:  # pylint: disable=too-few-public-methods
    """REST constants, for rest calls used in many sites"""

    ITEMS = "items"


class REST_ERROR:  # pylint: disable=too-few-public-methods
    """REST error constants"""

    DETAIL = "detail"


class API_CONSTANTS:  # pylint: disable=too-few-public-methods
    """API constants"""

    JOB_ID = "job_id"
    USER_ID = "user_id"
    SAVED_EXPERIMENT_ID = "saved_experiment_id"
    RUNCARD_ID = "runcard_id"
    FAVOURITE = "favourite"
    FAVOURITES = "favourites"
